class CreateSubpartida < ActiveRecord::Migration
  def self.up
    create_table :subpartida do |t|
      t.string :nombre
      t.integer :proyecto_id
      t.timestamps

    end
    add_column :presupuesto, :subpartida_id, :integer
    add_column :gasto, :subpartida_id, :integer

    rename_column :presupuesto, :subpartida, :concepto
    remove_column :gasto, :presupuesto_id
    File.open('app/models/presupuesto.rb', 'w') do |model|  
      model.puts  <<-__EOI
#--
#
#################################################################################
#Copyright (C) 2010 CENATIC y IEPALA
#Centro Nacional de Referencia de Aplicación de las TIC basadas en Fuentes Abiertas, Spain. 
#Instituto de Estudios Politicos para America Latina y Africa, Spain.
#
#This file is part of GONG. 
#
#GONG is free software; you can redistribute it and/or modify it under the 
#terms of the GNU General Public License as published by the Free Software Foundation;
#either version 3 of the License, or (at your option) any later version.
#
#
#GONG is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
#without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#See the GNU General Public License for further details, currently published at
#http://www.gnu.org/copyleft/gpl.html or in the gpl.txt in the licence folder of this distribution. 
#
#You should have received a copy of the GNU General Public License along with
#this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
#Street, Fifth Floor, Boston, Dt. 02110-1301, USA. 
#
#Date: 22 Abril 2010 
#Authors: Pascal Carrie (IEPALA), Jaime Ortiz Soriano (Semilla de Software Libre)
#You may contact the copyright holder at: 
#Fundación CENATIC. Web: http://www.cenatic.es. Email: info@cenatic.es
#Fundación IEPALA. Web: http://www.iepala.es. Email: iepala@iepala.es
#################################################################################
#
#++

# Clases del modelo que gestiona la entidad Presupuesto.
class Presupuesto < ActiveRecord::Base
  untranslate_all
  belongs_to :moneda
  belongs_to :partida
  belongs_to :libro
  belongs_to :agente
  belongs_to :subpartida

  has_many :presupuesto_x_proyecto, :dependent => :destroy
  has_many :presupuesto_x_agente, :dependent => :destroy
  has_many :presupuesto_x_actividad, :dependent => :destroy
  has_many :presupuesto_detallado, :dependent => :destroy
  has_many :tasa_cambio_presupuesto, :dependent => :destroy
  has_many :proyecto, :through => :presupuesto_x_proyecto

  validates_presence_of :moneda, :message => _("Moneda") + _(" no es válida.")  
  validates_presence_of :agente, :message => _("Agente") + _(" no es válido.")
  validates_presence_of :importe, :message => _("Importe") + _(" no pueden estar vacío.")
  validates_presence_of :partida, :message => _("Partida") + _(" no es válida.")
  validates_presence_of :subpartida, :message => _("Subartida") + _(" no es válida.")
  validate :fechas
  validate :concepto_unico

  def concepto_unico
    proyectos = self.proyecto
    proyectos.each do |proyecto|
      proyecto.presupuesto_x_proyecto.each do |pxp|
        unless pxp.presupuesto_id == self.id
          errors.add("concepto", _("Concepto repetido")) if pxp.presupuesto.concepto == self.concepto
        end
      end
    end
  end

  def subpartida_nombre
    subpartida.nombre if subpartida
  end 

  def subpartida_nombre=(nombre)
    self.subpartida=Subpartida.find_or_create_by_nombre(nombre) unless nombre.blank?
  end 

  # Comprueba que fecha inicio es mayor que fecha fin
  def fechas
    return errors.add("Fechas", _("Fecha fin (" + self.fecha_fin.to_s + "debe ser mayor o igual que fecha inicio (" + self.fecha_inicio.to_s + ".")) unless self.fecha_fin >= self.fecha_inicio
  end

  # Comprueba que las fechas están dentro de la etapa seleccionada.
  def comprobar_fechas_etapa  etapa
    unless self.fecha_inicio >= etapa.fecha_inicio and self.fecha_fin <= etapa.fecha_fin
      errors.add('Fechas', _("Las fechas del presupuesto no se encuentran dentro la etapa."))
      return false     
    else
      return true
    end
  end


  # Financiadores del presupuesto para un determinado proyecto
  def financiador proyecto
    presupuesto_x_agente.find :all, :conditions => {"proyecto_id" => proyecto.id}
  end
  # Activiadades del presupuesto para un determinado proyecto  
  def actividad proyecto
    presupuesto_x_actividad.find :all, :conditions => {"proyecto_id" => proyecto.id}
  end
  # Importe * el porcentaje asignado del presupuesto a un determinado proyecto.
  def importe_x_proyecto proyecto
    self.importe= self.importe * (presupuesto_x_proyecto.detect {|pxp| pxp.proyecto_id == proyecto.id}).porcentaje
  end
  
  # Devuelve la tasa de cambio para un proyecto y una financiación
  # se utilisa en tasa_cambio/tasas_presupuesto
  def tasa_cambio proyecto, financiacion
    return tasa_cambio_presupuesto.detect {|tcp| tcp.proyecto_id == proyecto and tcp.financiacion_id == financiacion}
  end

  #Metodo especifico creado para el ruport y la exportacion
  def moneda_abreviatura
    return Moneda.find(self.moneda_id).abreviatura if self.moneda_id
  end

  #Metodo especifico creado para el ruport y la exportacion
  def partida_codigo_nombre_aborrar 
    return partida.codigo_nombre if self.partida_id
  end

  #Metodo especifico creado para el ruport y la exportacion
  def agente_nombre
    return Agente.find(self.agente_id).nombre if self.agente_id
  end

 # Meses que comprende el presupuesto.
  def meses
    fecha = self.fecha_inicio; i = 0
    while  fecha < self.fecha_fin; fecha = fecha >> 1; i += 1;  end
    return i
  end

  # Presupuesto para una determinada etapa, proyecto y partida
  def self.por_etapa_proyecto_partida etapa, proyecto, partida
    find :all, :include => [:presupuesto_x_proyecto], :conditions => {"presupuesto.fecha_inicio" => etapa.fecha_inicio..etapa.fecha_fin, 
                                                                      "presupuesto_x_proyecto.proyecto_id" => proyecto.id,  
                                                                      "presupuesto.partida_id" => partida }
  end
  # Presupuesto para una determinada etapa, agente y partida
  def self.por_etapa_agente_partida etapa, agente, partida
    find :all, :conditions => {"presupuesto.fecha_inicio" => etapa.fecha_inicio..etapa.fecha_fin, "presupuesto.agente_id" => agente.id, "presupuesto.partida_id" => partida}
  end

  # Actualiza el listado de agentes con los que se ha asociado el presupuesto para un determinado proyecto.
  def actualizar_presupuesto_x_agente listado, proyecto
    financiador(proyecto).each {|f| f.destroy}
    listado.each do |key, value|
      pxa = presupuesto_x_agente.create(value)
      errors.add("", pxa.errors.inject('') {|total, e| total + e[1] + " " }) unless pxa.errors.empty?
    end        
    if listado.inject(0) {|suma, value| suma + value.pop[:porcentaje].to_f} != 1
      errors.add(_('Porcentajes'), _("El porcentaje de las financiaciones debe sumar 100."))        
    end
  end

  # Actualiza el listado de actividades con los que se ha asociado el presupuesto para un determinado proyecto.
  def actualizar_presupuesto_x_actividad listado, proyecto
    actividad(proyecto).each {|f| f.destroy}
    listado.each do |key, value|
      pxa = presupuesto_x_actividad.create(value)
      errors.add("", pxa.errors.inject('') {|total, e| total + e[1] + "<br>" }) unless pxa.errors.empty?
    end        
    if  listado.inject(0) {|suma, value| suma + value.pop[:porcentaje].to_f} != 1
      errors.add(_('Porcentajes'), _("El porcentaje de las actividades asociadas al presupuesto debe sumar 100."))        
    end
  end

  # Actualiza el detalle del presupuesto.
  def actualizar_presupuesto_detallado listado
    presupuesto_detallado.clear
    listado.each do |mes, value|
       presupuesto_detallado.create(value) unless value[:importe] == ""
    end
    errors.add(_('Porcentajes'),_("La suma del detalle no coincide con la importe de la linea del presupuesto. Vuelva a editar el detalle para corregirlo.")) unless self.importe == presupuesto_detallado.inject(0) {|sum,p| sum+p.importe}
  end

  # Actualiza los proyectos a los que se ha asociado el gasto.
  def actualizar_presupuesto_x_proyecto listado
    presupuesto_x_proyecto.clear
    if  listado.inject(0) {|suma, value| suma + value.pop[:porcentaje].to_f} == 1
      listado.each do |key, value|
        pxp = presupuesto_x_proyecto.create(value)
        errors.add("", pxp.errors.inject('') {|total, e| total + e[1] + " " }) unless pxp.errors.empty?
      end        
    else
      errors.add(_('Porcentajes'), ("El porcentaje de los proyectos asociados al presupuesto debe sumar 100. No se guardan los datos."))        
    end
  end

  # utilizado solamente en el partial comunes/_activ_financ.rhtml que ya no se usa
  def comprobar_financiadores_aborrar proyecto
    if financiador(proyecto).inject(0) {|suma, f| suma + f.porcentaje} != 1 
      return "financiador_rojo"
    else
      return "financiador"
    end    
  end

  def comprobar_actividades_aborrar proyecto
    if actividad(proyecto).inject(0) {|suma, f| suma + f.porcentaje} != 1
      return "actividades_rojo"
    else
      return "actividades"
    end    
  end

  # Comprueba si el detalle del presupuesto se ha configurado adecuadamente.
  def comprobar_presupuesto_detallado
    if presupuesto_detallado.inject(0) {|suma, p| suma + p.importe} != self.importe 
      return "alerta", _("¡Atencion! El detalle del presupuesto no es correcto")
    else
      return "temporal", _("Editar detalle de presupuesto")
    end    
  end

  # Comprueba si el presupuesto esta asociado correctamente a proyectos.
  # solamente utilizado en app/views/presupuesto_agentes/listado.rhtml:<% imagen, titulo = presupuesto.comprobar_proyectos %>
  def comprobar_proyectos
    if presupuesto_x_proyecto.inject(0) {|suma, pxp| suma + pxp.porcentaje} != 1
      return "alerta", _("¡Atención! Falta por asginar proyectos al presupuesto")
    else
      return "proyectos", _("Editar proyectos asociados al presupuesto")
    end    
  end

  #--
  # FINDERS complicados... Mas alla de ActiveRecordBase
  # ++

  # Busqueda creada manualmente para los detalles de busquedas especificas que no cubre el find habitual de ActiveRecord.
  def self.busqueda campos, condiciones, asociacion, agrupado, ordenado
    consulta = " SELECT " + campos + " FROM presupuesto  "
    consulta += asociaciones asociacion
    consulta += " WHERE " + merge_conditions(condiciones) if condiciones
    consulta += " GROUP BY " + agrupado if agrupado
    consulta += " ORDER BY " + ordenado if ordenado
    return find_by_sql consulta 
  end
  
  # Cutrada para sustituir el include para el metodo busqueda.
  def self.asociaciones asociacion
    asociacion.inject(' ') do |suma, elemento|
      case elemento
        when :presupuesto_x_proyecto
        suma + ' LEFT JOIN presupuesto_x_proyecto on presupuesto_x_proyecto.presupuesto_id = presupuesto.id'
        when :tasa_cambio_presupuesto
        suma + ' LEFT JOIN tasa_cambio_presupuesto on tasa_cambio_presupuesto.presupuesto_id = presupuesto.id'
        when :presupuesto_x_agente
        suma + ' LEFT JOIN presupuesto_x_agente on presupuesto_x_agente.presupuesto_id = presupuesto.id'
        when :presupuesto_x_actividad
        suma + ' LEFT JOIN presupuesto_x_actividad on presupuesto_x_actividad.presupuesto_id = presupuesto.id'
        when :partida_x_partida_financiacion
        suma + ' LEFT JOIN partida_x_partida_financiacion on partida_x_partida_financiacion.partida_id = presupuesto.partida_id'
        when :partida_financiacion
        suma + ' LEFT JOIN partida_financiacion on partida_financiacion.id = partida_x_partida_financiacion.partida_financiacion_id'
        when :presupuesto_detallado
        suma + ' LEFT JOIN presupuesto_detallado on presupuesto_detallado.presupuesto_id = presupuesto.id'
      end
    end
  end

  # Metodo que iguala algunos valores basicos para procesos de insercion de datos.
  def self.datos_basicos_igualar presupuesto, presupuesto_anterior
    presupuesto.partida = presupuesto_anterior.partida
    presupuesto.fecha_inicio = presupuesto_anterior.fecha_inicio
    presupuesto.fecha_fin = presupuesto_anterior.fecha_fin
    presupuesto.agente_id = presupuesto_anterior.agente_id
    presupuesto.moneda_id = presupuesto_anterior.moneda_id

  end

end
__EOI
    end
    Dispatcher.remove_class Presupuesto
    load "app/models/presupuesto.rb"
    File.open('app/models/gasto.rb', 'w') do |model|  
      model.puts  <<-__EOI
#--
#
#################################################################################
#Copyright (C) 2010 CENATIC y IEPALA
#Centro Nacional de Referencia de Aplicación de las TIC basadas en Fuentes Abiertas, Spain. 
#Instituto de Estudios Politicos para America Latina y Africa, Spain.
#
#This file is part of GONG. 
#
#GONG is free software; you can redistribute it and/or modify it under the 
#terms of the GNU General Public License as published by the Free Software Foundation;
#either version 3 of the License, or (at your option) any later version.
#
#
#GONG is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
#without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#See the GNU General Public License for further details, currently published at
#http://www.gnu.org/copyleft/gpl.html or in the gpl.txt in the licence folder of this distribution. 
#
#You should have received a copy of the GNU General Public License along with
#this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
#Street, Fifth Floor, Boston, Dt. 02110-1301, USA. 
#
#Date: 22 Abril 2010 
#Authors: Pascal Carrie (IEPALA), Jaime Ortiz Soriano (Semilla de Software Libre)
#You may contact the copyright holder at: 
#Fundación CENATIC. Web: http://www.cenatic.es. Email: info@cenatic.es
#Fundación IEPALA. Web: http://www.iepala.es. Email: iepala@iepala.es
#################################################################################
#
#++

# gasto

class Gasto < ActiveRecord::Base

  untranslate_all

  
  belongs_to :moneda
  belongs_to :libro
  belongs_to :partida
  belongs_to :subpartida

  has_many :gasto_x_proyecto, :dependent => :destroy
  has_many :gasto_x_agente, :dependent => :destroy
  has_many :gasto_x_actividad, :dependent => :destroy
  has_many :gasto_x_transferencia, :dependent => :destroy
  has_many :tasa_cambio_gasto, :dependent => :destroy
  #has_many :gasto_identificacion, :dependent => :destroy
  
  has_and_belongs_to_many :transferencia, :join_table =>  "gasto_x_transferencia"

  validates_presence_of :importe, :message => _("Importe") + _(" no pueden estar vacío.")
  validates_presence_of :moneda_id, :message => _("Moneda") + _(" no pueden estar vacío.")
  validates_presence_of :partida_id, :message => _("Partida") + _(" no pueden estar vacío.")
  validates_presence_of :libro_id, :message => _("Cuenta") + _(" no pueden estar vacío.")  
  validates_presence_of :fecha,  :message => _("Fecha") + _(" no pueden estar vacío.")  
  validates_associated :libro, :message =>  _("La cuenta asociada no es correcta.")  

# De momento comentamos los temas relacionados con informes hasta que tengamos claro el sentido que va a tomar.
#  before_save :fecha_y_fecha_informe
#  def fecha_y_fecha_informe
#    self.fecha_informe = self.fecha if self.fecha_informe.nil?
#  end

  def subpartida_nombre
    subpartida.nombre if subpartida
  end 

#   def subpartida_nombre=(nombre)
#     self.subpartida=Subpartida.find_or_create_by_nombre(nombre) unless nombre.blank?
#   end 

  # Comprueba que la fecha esta dentro de la etapa selecionada. 
  def comprobar_fecha_etapa etapa
    if self.fecha and self.fecha > etapa.fecha_inicio and self.fecha < etapa.fecha_fin
      return true
    else
      errors.add(_("Fecha"), _("La fecha debe estar dentro de la etapa")) ; return false
    end
  end

  # Devuelve los financiadores del gasto para un determinado proyecto   
  def financiador proyecto
     gasto_x_agente.find :all, :conditions => {"proyecto_id" => proyecto.id}
  end

  # Devuelve los actividades del gasto para un determinado proyecto
  def actividad proyecto
     gasto_x_actividad.find :all, :conditions => {"proyecto_id" => proyecto.id}
  end  

   # Moneda del gasto
   def moneda
     return self.moneda_id ? Moneda.find(self.moneda_id) : nil
   end

   #Metodo especifico creado para el ruport y la exportacion
   def moneda_abreviatura
     return Moneda.find(self.moneda_id).abreviatura if self.moneda_id
   end

   #Metodo especifico creado para el ruport y la exportacion
   def partida_codigo_nombre_aborrar
     return partida.codigo_nombre if self.partida_id
   end

   # importe * el porcentaje con el que se ha asociado el gasto al proyecto
   def importe_x_proyecto proyecto
 #    if gasto_x_proyecto.size > 1
     self.importe = self.importe * (gasto_x_proyecto.detect {|gxp| gxp.proyecto_id == proyecto.id}).porcentaje
 #    end
   end
  
   # Actualiza el listado de agentes con los que se ha asociado el gasto para un determinado proyecto.
   def actualizar_gasto_x_agente listado, proyecto
     financiador(proyecto).each {|f| f.destroy}
     listado.each do |key, value|
       gxa = gasto_x_agente.create(value)
       errors.add("", gxa.errors.inject('') {|total, e| total + e[1] + "<br>" }) unless gxa.errors.empty?
     end        
     if  listado.inject(0) {|suma, value| suma + value.pop[:porcentaje].to_f} != 1
       errors.add( _('Porcentajes'), _("El porcentaje de las financiaciones debe sumar 100."))        
     end
   end

   # Actualiza el listado de actividades con los que se ha asociado el gasto para un determinado proyecto.
   def actualizar_gasto_x_actividad listado, proyecto
     actividad(proyecto).each {|f| f.destroy}
     listado.each do |key, value|
       gxa = gasto_x_actividad.create(value)
       errors.add( "", gxa.errors.inject('') {|total, e| total + e[1] + "<br>" } ) unless gxa.errors.empty?
     end        
     if  listado.inject(0) {|suma, value| suma + value.pop[:porcentaje].to_f} != 1
       errors.add( _('Porcentajes'), _("El porcentaje de las actividades asociadas al gasto debe sumar 100.") )        
     end
   end

   # Actualiza los proyectos a los que se ha asociado el gasto.
   def actualizar_gasto_x_proyecto listado
    gasto_x_proyecto.clear
    if  listado.inject(0) {|suma, value| suma + value.pop[:porcentaje].to_f} == 1
      listado.each do |key, value|
        gxp = gasto_x_proyecto.create(value)
        errors.add( "", gxp.errors.inject('') {|total, e| total + e[1] + "<br>" } ) unless gxp.errors.empty?
      end        
    else
      errors.add( _('Porcentajes'), _("El porcentaje de los proyectos asociados al gasto debe sumar 100. No se guardan los datos.") )        
    end
  end

  # Devuelve la tasa de cambio para un determinado proyeto y una determinada financiación.
  def tasa_cambio proyecto, financiacion
    return tasa_cambio_gasto.detect {|tcg| tcg.proyecto_id == proyecto and tcg.financiacion_id == financiacion}
  end

  # Devuelve la identificacion del gasto para un determinado proyeto y una determinada financiación.
  def identificacion_gasto_aborrar proyecto, financiacion
    return gasto_identificacion.detect {|ig| ig.proyecto_id == proyecto and ig.financiacion_id == financiacion}
  end

  # Comprueba si el gasto esta asociado correctamente a proyectos.
  def comprobar_proyectos
    if gasto_x_proyecto.inject(0) {|suma, pxp| suma + pxp.porcentaje} != 1
      return "alerta", _("¡Atención! Falta por asignar proyectos a gasto")
    else
      return "proyectos", _("Editar proyectos asociados a gasto")
    end    
  end

  #--
  # FINDERS complicados... Mas alla de ActiveRecordBase
  # ++

  # Busqueda creada manualmente para los detalles de busquedas especificas que no cubre el find habitual de ActiveRecord.
  def self.busqueda campos, condiciones, asociacion, agrupado, ordenado
    consulta = " SELECT " + campos + " FROM gasto  "
    consulta += asociaciones asociacion
    consulta += " WHERE " + merge_conditions(condiciones) if condiciones
    consulta += " GROUP BY " + agrupado if agrupado
    consulta += " ORDER BY " + ordenado if ordenado
    return find_by_sql consulta 
  end
  
  # Cutrada para sustituir el include para el metodo busqueda.
  def self.asociaciones asociacion
    asociacion.inject(' ') do |suma, elemento|
      case elemento
        when :libro
        suma + ' LEFT OUTER JOIN libro on gasto.libro_id = libro.id'
        when :gasto_x_proyecto
        suma + ' LEFT OUTER JOIN gasto_x_proyecto on gasto_x_proyecto.gasto_id = gasto.id'
        when :tasa_cambio_gasto
        suma + ' LEFT OUTER JOIN tasa_cambio_gasto on tasa_cambio_gasto.gasto_id = gasto.id'
        when :gasto_x_agente
        suma + ' LEFT OUTER JOIN gasto_x_agente on gasto_x_agente.gasto_id = gasto.id'
        when :gasto_x_actividad
        suma + ' LEFT OUTER JOIN gasto_x_actividad on gasto_x_actividad.gasto_id = gasto.id'
        when :partida_x_partida_financiacion
        suma + ' LEFT OUTER JOIN partida_x_partida_financiacion on partida_x_partida_financiacion.partida_id = gasto.partida_id'
        when :partida_financiacion
        suma + ' LEFT OUTER JOIN partida_financiacion on partida_financiacion.id = partida_x_partida_financiacion.partida_financiacion_id'
        #when :gasto_identificacion
        #suma + ' LEFT OUTER JOIN gasto_identificacion on gasto_identificacion.gasto_id = gasto.id'
      end
    end
  end

  # Metodo que iguala algunos valores basicos para procesos de insercion de datos.
  def self.datos_basicos_igualar gasto, gasto_anterior
    gasto.partida = gasto_anterior.partida
    gasto.fecha = gasto_anterior.fecha
    gasto.libro_id = gasto_anterior.libro_id
    gasto.moneda_id = gasto_anterior.moneda_id
    gasto.subpartida_id = gasto_anterior.subpartida_id    
  end

end
__EOI
    end
    Dispatcher.remove_class Gasto
    load "app/models/gasto.rb"
  end

  def self.down
    add_column :presupuesto, :subpartida, :string
    add_column :gasto, :presupuesto_id, :integer
    rename_column :presupuesto, :concepto, :subpartida
    remove_column :presupuesto, :subpartida_id
    remove_column :gasto, :subpartida_id
    drop_table :subpartida
  end
end
